# -*- coding: utf-8 -*-
import wtforms

__all__ = ("EnumSelectField",)


def _simple_i18n(value):
    return str(value)


_unset_value = object()


class EnumSelectField(wtforms.SelectField):
    def __init__(self, enum_type, i18n=_simple_i18n, allow_none=False, *args, **kvargs):
        super(EnumSelectField, self).__init__(*args, **kvargs)
        self._enum_type = enum_type
        self.choices = []

        if allow_none:
            self.choices.append(allow_none if isinstance(allow_none, tuple) else (u"", u"(Please select)",))

        for element in self._enum_type.__elements__:
            self.choices.append((str(element.ordinal()), i18n(str(element)),))

    def as_constant(self):

        if not self.data or self.data == "None" or self.data is _unset_value:
            raise ValueError(
                u"invalid identifier for the registered type (data=%s, enum_type=%s)" % (self.data, self._enum_type))
        else:
            return self._enum_type.__elements__[int(self.data)]



# -*- coding: utf-8 -*-

def forward_property(name, data_member="_data"):
    """ a simple property-forwarder
    this forwarder is used for accessing property-values which are contained in a separate container


    class Foo(object):
        def __init__(self, data):
            self._data = data

        id = forward_property("id")

    class Foo(object):
        def __init__(self, data):
            self._bar = data

        id = forward_property("id", data_member="bar")

    data = dict(id=1)

    foo1 = Foo(data)
    foo2 = Foo(data)

    print foo1.id
    >> 1
    print foo2.id
    >> 1
    """
    def _obj_from_chain(chain_key, current_dom):
        for attr_name in chain_key.split("."):
            current_dom = getattr(current_dom, attr_name)
        return current_dom

    def getter(self): return getattr(_obj_from_chain(data_member, self),name)
    def setter(self, value): setattr(_obj_from_chain(data_member, self), name, value)
    return property(getter, setter)




# -*- coding: utf-8 -*-
"""

"""

import StringIO
import collections, logging, os, re

log = logging.getLogger(__name__)

class Filter(object):
    def validate(self):
        raise Exception(u"nothing implemented here")

# Image Type / File type filters
class Jpeg(Filter):
    pass

class Gif(Filter):
    pass

class Png(Filter):
    pass

class Resize(Filter):
    def __init__(self, width, heighjt):
        pass

class Watermark(Filter):
    def __init__(self, image_stream):
        pass

class Pdf(Filter):
    pass

class Zip(Filter):
    pass

class Images(Filter):
    pass


# -*- coding: utf-8 -*-
""" the octavius module provides simply a way to store and relocate files
by this, it is producing a handler-object which represents one or many
versions of the same file

the specific handler class is responsible for creating and identifing
it multiple versions of it one master-file

the defined and used storage engine on the other hand is only responsible
for storing the physical information of a file into a unspecified storage
and providing a information-object with all the required data to remapping
the versions for an @AssetHandler


usage:

# define a file handler
>>> class GalleryImage(AssetHandler):
>>>     thumbnail = Image(filters=())


# create a default storage engine
>>> engine = DefaultStorage(credentials)

# create a manager to handle this type
>>> manager = AssetManager(GalleryImage, engine)

>>> image_handler = manager.create(StreamIO, filename)

# every handler a the original, never touched, file itself as "master"-property
>>> print image_handler.master
FileInfo()

>>> print image_handler.master.file.original_filename
some_filename.jpg

>>> print image_handler.master.file.path_to_file
/path/to/physical/file

# you can address a specific version by simple using the defined property
>>> print image_handler.thumbnail
FileInfo()

>>> print image_handler.thumbnail.file.original_filename
some_filename.jpg

>>> print image_handler.thumbnail.file.path_to_file
/path/to/physical/file

# you can also iterate through the available versions. this do not include
# the master-reference
>>> for image in image_handler:
>>>     print image
>>>     print image.original_filename
>>>     print image.path_to_file

# an AssetHandler has also it's own ident
>>> print image_handler.ident()
"""



import StringIO
import collections, logging, os, re

import filters as filt

log = logging.getLogger(__name__)

def extension():

    table = {
        'text/plain': '.txt',
        'text/html': '.html',
        'text/xml': '.xml',
        'text/css': '.css',
        'text/csv': '.csv',
        'application/postscript': '.ps',
        'application/vnd.ms-excel': '.xsl',
        'application/xml': '.xml',
        'application/msword': '.doc',
        'application/pdf': '.pdf',
        'image/png': '.png',
        'image/jpeg': '.jpg',
        'image/gif': '.gif',
        'image/tiff': '.tif',
        }

    def resolve(type):
        return table.get(type, ".data")

    return resolve


extension = extension()

class FileAccessHandler:
    def __init__(self, path_to_file):
        self.path_to_file = path_to_file
        
    def __enter__(self):
        try:
            self.pointer = open(self.path_to_file, "rw")
            log.debug(u"opened a existing file (file=%s)", self.path_to_file)
        except IOError as e:
            self.pointer = open(self.path_to_file, "w+b")
            log.debug(u"create a new file (file=%s)", self.path_to_file)
            
        return self.pointer
    
    def __exit__(self, type, value, traceback):
        self.pointer.close()

class FileInfo(object):
    """ Abstract base class for all file info
    every storage-engine should implement a version of this for it's own use
    """
    
    ident = None
    original_filename = None
    mime_type = None
    path_to_file = None
    full_path_to_file = None
    
    def file_handler(self):
        """ returns a access handler to the current file in the filesystem
        """
        dirname = os.path.dirname(self.full_path_to_file)
        
        if not os.path.isdir(dirname):
            log.debug(u"create new directory for %s" % dirname)
            os.makedirs(dirname)

        return FileAccessHandler(self.full_path_to_file)
    
    def __repr__(self):
        return u"<FileInfo(ident=%s, original_filename=%s, mime_type=%s, path_to_file=%s, full_path_to_file=%s)>" % (self.ident, 
                                                                                                                     self.original_filename, 
                                                                                                                     self.mime_type, 
                                                                                                                     self.path_to_file, 
                                                                                                                     self.full_path_to_file)

class Unbound(object):
    _args = None
    _kvargs = None
    
    def __init__(self, version_clazz, *args, **kwargs):
        self._version_clazz = version_clazz
        self._args = args
        self._kwargs = kwargs
    
    def bind(self, instance):
        return self._version_clazz(bind_to=instance, *self._args, **dict(self._kwargs))

class FileVersion(object):
    """ FileVersion´s are templates which define how byte-data should be with proceeded
    The real byte/file-informations are accessable through the file()-method
    """
    _allowed_ = None
    
#    def __call__(self, file_info):
#        """ initializing a file-version with the matching FileInfo 
#        """
#        self._file_info = file_info
    def __new__(cls, *args, **kwargs):
        if "bind_to" in kwargs.keys():
            return super(FileVersion, cls).__new__(cls)
        else:
            return Unbound(cls, *args, **kwargs)

    def __init__(self, bind_to=None):
        self.instance = bind_to
        self._file_info = None
    
    def update(self, engine, source_info):
        """ creating a copy of the file-data, specified in source_info and create a new version of it
        
        engine : StorageEngine
        source_info : pytoolbelt.asset_mgmt.handler.FileInfo
        """
        file_content = None
        
        with source_info.file_handler() as iostream:
            file_content = iostream.read()
            
        self._file_info = engine.store(file_content, source_info.original_filename, source_info.mime_type)
    
    @property
    def file(self):
        """ returns the real physical representation of the current file-version
        """
        return self._file_info

class Image(FileVersion):
    """ Image is a superset of FileVersion, which contains a unmutable range of accepted filetypes
    """
    def __init__(self, filters=None, *args, **kwargs):
        super(Image, self).__init__(*args, **kwargs)
        self._allowed_ = (filt.Jpeg(), filt.Gif(), filt.Png(),)

class Glue(object):
    
    def __init__(self, reference_key, unbound):
        self._reference_key = reference_key
        self._unbound = unbound
    
    def __get__(self, instance, owner):
        if instance is None: return self._unbound
        if not self._reference_key in instance.__dict__.keys():
            instance.__dict__[self._reference_key] = self._unbound.bind(instance)
            
        return instance.__dict__.get(self._reference_key)

class MetaAssetHandler(type):
    def __new__(cls, name, bases, cdict):
        """ creating bound instances of currently unbound FileVersion-objects into the new instance of this class
        """
        ndict = dict(cdict)
        bindings = dict()
        
        
        for k,v in cdict.iteritems():
            if isinstance(v, Unbound):
                
                glue = Glue(u"__%s_%s" % (name,k), v)
                bindings[k] = ndict[k] = glue
        
        ndict["__bindings"] = bindings
        self = type.__new__(cls, name, bases, ndict)
        return self

class AssetHandler(object):
    __metaclass__ = MetaAssetHandler
    
    master = FileVersion()

    def __init__(self, storage_engine, master):
        self.master.update(storage_engine, master)
        self._se = storage_engine
    
    @property
    def display_name(self):
        raise Exception("not implemented yet")
        
    @property
    def ident(self):
        """ the primary ident for a AssetHandler
        
        it is regardless, if the ident of the master or any
        subversion will be used, because the original master
        will be determined based on the ident-properties.
        but for simplifications we should address the
        master-asset
        """
        return self.master.file.ident

    def update(self):
        """ create or update all physical file-versions of the the master-asset
        """
        # check ident-duplications in all versions
        # iterate through versions
        # execute filters
        # determine unique ident for each version
        # store resulting stream
#        for k,file_version in self.__bindings.iteritems():
#            file_version.update(self._se, self.master.file)
        pass



class AssetManager(object):
    def __init__(self, handler_clazz, storage_engine):
        self.handler_clazz = handler_clazz
        self.storage_engine = storage_engine

    def create(self, file_content, filename, mime_type=None):
        file_info = self.storage_engine.store(file_content, filename, mime_type)
        handler = self.handler_clazz(self.storage_engine, file_info)
        handler.update()
        return handler

    def load(self, ident):
        master_asset = self.storage_engine.load(ident)
        return self.handler_clazz(self.storage_engine, master_asset)


if __name__ == "__main__":
    
    master = FileInfo()
    
    class Image(AssetHandler):
        default = FileVersion()
        large = FileVersion()
    
    print Image.master
    print Image.default
    
    handler1 = Image(None, master)
    handler2 = Image(None, master)
    
    print  "handler1 is handler 2 (master)", handler1.master is handler2.master
    print  "handler1 is handler 2 (default)", handler1.default is handler2.default
    
    print "handler1: ", handler1.master,  
    print "handler1: ", handler1.default
    print "handler2: ", handler2.master
    print "handler2: ", handler2.default
    
    
    
    

        
        


# -*- coding: utf-8 -*-

""" This module provides simple enum support. A little bit of meta class


usage:

class Foo(enumerated):
    FOO = enum()

class Bar(enumerated):
    BAR = enum(value=1)

    def __setup__(self, value):
        self.value = value


print Foo.FOO
print Bar.BAR.value
"""

__all__ = ['enumerated', 'enum']


class _TagProperty(object):
    counter = 0

    def __init__(self, args, keys):
        super(_TagProperty, self).__init__()
        self._ordinal = _TagProperty.counter
        _TagProperty.counter += 1
        self._init_args = args
        self._init_keys = keys


class EnumClass(type):
    def __new__(klass, name, bases, defines):

        all_props = dict()
        tag_props = list()

        for key, value in defines.iteritems():
            if isinstance(value, _TagProperty):
                tag_props.append((key, value))
            else:
                all_props[key] = value

        self = type.__new__(klass, name, bases, all_props)

        elements = list()

        for key, value in tag_props:
            constant = self.__new__(self, key, value._ordinal, value._init_args, value._init_keys)
            setattr(self, key, constant)
            elements.append(constant)

        elements.sort(lambda s, t: cmp(s.__ordinal__, t.__ordinal__))

        for index, value in enumerate(elements):
            value.__ordinal__ = index

        self.__elements__ = tuple(elements)
        return self

    def get(self, name, default=None):
        value = getattr(self, name, None)
        if isinstance(value, self): return value
        return default

    def __getitem__(self, name):
        value = getattr(self, name, None)
        if isinstance(value, self): return value
        raise KeyError(name)


def unpickler(type, name):
    return type[name]


class enumerated(object):
    """Base class for enumeration types
    
    This class is the base class for all enumeration types. You define a new
    enumeration by inheriting from this class. Note, that you should never inherit
    from any enum class except this one.
    
    Actual instances of this class are usually only created by assignments of
    `enum()` to class properties in the class suite of a subclass definition.
    
    This class provides a total ordering of its instances, which relies on the
    implicit `ordinal` value assigned to each instance upon construction. Note,
    that two instances of this class are comparable (with respect to `<`, `<=`,
    `>`, `>=`) if (and only if) they are instances of the very same subclass.
    
    Equality is defined in terms of identity for this class.
    
    Example:
    
        >>> class Scope(enumerated):
        ...
        ...    LOCAL = enum()
        ...    CLASS = enum()
        ...    GLOBAL = enum()
        
    The resulting class is an enum, which defines three constant members, each
    of which is an instance of class `Scope`:
    
        >>> Scope.LOCAL
        Scope.LOCAL
        
    Enum constants are usable as indices, for example into lists or tuples:
    
        >>> data = [1, 2, 3]
        >>> data[Scope.CLASS]
        2
        
    The first defined constant has the implicit ordinal value `0`, the second one
    has the value `1`, and so on.
    
    You can associate arbitrary information to with each enum constant; simply
    pass keyword arguments to the `enum` function:
    
        >>> class Instruction(enumerated):
        ...
        ...    __slots__ = ('stack_effect', 'ppname')
        ...
        ...    PUSH = enum(ppname = 'push', stack_effect = 1)
        ...    POP = enum(ppname = 'pop', stack_effect = -1)
        ...    DUP = enum(ppname = 'duplicate', stack_effect = 1)
        ...    SWAP = enum(ppname = 'swap', stack_effect = 0)
        ...
        ...    def __setup__(self, ppname = None, stack_effect = 0):
        ...        self.ppname = ppname
        ...        self.stack_effect = stack_effect
        
    which is made available to the `__setup__` method (which is called instead 
    Python's standard `__init__` when an instance of an enumerated class is
    created.)
    """

    __metaclass__ = EnumClass
    __slots__ = ('__weakref__', '__name__', '__ordinal__',)

    def __new__(klass, name, ordinal, args=(), keys={}):
        self = object.__new__(klass)
        self.__name__ = name
        self.__ordinal__ = ordinal
        self.__setup__(*args, **keys)
        return self

    def __setup__(self):

        """Initializer for enum constants
        
        This method is called instead of Python's standard `__init__`, when an
        enumeration constant value is constructed. It receives the arguments passed
        to the `enum` function when the defining class body was executed. 
        Subclasses may define additional set-up code here.
        """

        pass

    def ordinal(self):

        """Obtains this constant's ordinal value
        
        Every enumeration constant has an ordinal value, which can be obtained by
        calling this method. The ordinal is implicitly defined by the definition
        order in the `class` statement, which constructed the enum.
        """

        return self.__ordinal__

    def name(self):

        """Obtains this constant's name
        
        Every enumeration constant has a name, which is implicitly defined by the
        name, the constant has been assigned to in the body of the `class` statement.
        This method returns the name thus given to the constant.
        """

        return self.__name__

    def __str__(self):
        return "%s.%s" % (self.__class__.__name__, self.__name__)

    def __repr__(self):
        return self.__str__()

    def __le__(self, other):
        if not isinstance(other, type(self)): return NotImplemented
        return self.__ordinal__ <= other.__ordinal__

    def __lt__(self, other):
        if not isinstance(other, type(self)): return NotImplemented
        return self.__ordinal__ < other.__ordinal__

    def __ge__(self, other):
        if not isinstance(other, type(self)): return NotImplemented
        return self.__ordinal__ >= other.__ordinal__

    def __gt__(self, other):
        if not isinstance(other, type(self)): return NotImplemented
        return self.__ordinal__ > other.__ordinal__

    def __index__(self):
        return self.__ordinal__

    def __reduce__(self):
        return unpickler, (type(self), self.__name__)


def enum(*args, **keys):
    """Define an enumeration constant within a `enum` class body
    
    This function should only be used in the body of a `class` suite defining
    a new enumeration. The value returned is a temporary placeholder for the
    enumeration constant being defined.
    
    Any arguments supplied to this function will be passed down to the new
    constant's `__setup__` method when the constant value is actually created.
    """

    return _TagProperty(args, keys)



# -*- coding: utf-8 -*-

# This module is part of onfo_octavius and is released under
# the MIT License: http://www.opensource.org/licenses/mit-license.php
#
# this enine is used by the onfo_ocatvius asset management module
# it describes, how a filestream will be saved to the filesystem
# and registered/referenced in a database
#

import logging, collections, sqlalchemy, datetime, transaction

import sqlalchemy.ext.declarative as sqla_decl
import sqlalchemy.orm as sqla_orm
from zope.sqlalchemy import ZopeTransactionExtension

import pytoolbelt.asset_mgmt.handler as core_handler

Base = sqla_decl.declarative_base()


log = logging.getLogger(__name__)

# The Ident-object provides the required information for determining 
# a unique, file-specific naming
Ident = collections.namedtuple("Ident", ("id", "parent_id", "str1",))
Credentials = collections.namedtuple("Credentials", ("storage_directory", "alchemy_session",))

def resolve_filepath(id, mime_type):
    id = id
    top_dir = "%02x" % (0xff & (id >> 8),)
    sub_dir = "%02x" % (0xff & (id >> 16),)
    file = "%012x%s" % ((0xff & id) | (id >> 24), core_handler.extension(mime_type))
    return "/".join((top_dir, sub_dir, file,))


class FileInfo(core_handler.FileInfo):
    """ FileInfo-Implementation for the DefaultStorageEngine
    
    all required file-infos will be provided by a sqlalchemy-dao
    and the engine-credentials 
    """
    def __init__(self, data, storage_directory):
        self._data = data
        self._storage_directory = storage_directory

        self.ident = Ident(data.id, data.parent_id, data.str1)
        self.original_filename = data.original_filename
        self.mime_type = data.mime_type

        self.path_to_file = resolve_filepath(data.id, data.mime_type)
        
        self.full_path_to_file = "/".join((self._storage_directory,self.path_to_file,))


class Asset(Base):
    __tablename__ = 'asset'
    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
    parent_id = sqlalchemy.Column(sqlalchemy.Integer, nullable=True)
    original_filename = sqlalchemy.Column(sqlalchemy.VARCHAR(255), nullable=False)
    mime_type = sqlalchemy.Column(sqlalchemy.VARCHAR(10), nullable=False)
    int1 = sqlalchemy.Column(sqlalchemy.INTEGER(2), nullable=True)
    str1 = sqlalchemy.Column(sqlalchemy.VARCHAR(100), nullable=True)
    created_on = sqlalchemy.Column(sqlalchemy.DATETIME, nullable=False, default=datetime.datetime.utcnow)
    modified_on = sqlalchemy.Column(sqlalchemy.DATETIME, nullable=False, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow)

    def __init__(self, original_filename, mime_type):
        self.original_filename = original_filename
        self.mime_type = mime_type

    def __repr__(self):
        return u"Asset(id=%s, parent_id=%s, str1=%s, mime_type=%s, )" % (self.id, self.parent_id, self.str1, self.mime_type)


class DefaultStorageEngine(object):
    def __init__(self, credentials):
        self.storage_directory = credentials.storage_directory
        self.alchemy_session = credentials.alchemy_session

    def store(self, stream, filename, mime_type, ident=Ident(None, None, None), str1=None, int1=None):
        log.info(u"register a asset to the database (filename=%s, mime_type=%s, ident=%s)" % (filename, mime_type, ident))
        data = Asset(filename, mime_type)
        
        if not str1 is None: data.str1 = str1
        if not int1 is None: data.int1 = int1
        
        with transaction.manager:
            self.alchemy_session.add(data)
        
        # TODO: save the file to the filesystem
        
        return FileInfo(data, self.storage_directory)

    def load(self, ident):
        data = None
        
        if ident.id is None:
            raise ValueError(u"ID-value can not be None (data=%s)"  % ident.id)
        
        with transaction.manager:
            query = self.alchemy_session.query(Asset)
            query = query.filter(Asset.id==ident.id)
            
            if query.count() == 1:
                return FileInfo(query.first(), self.storage_directory)
            
            log.info(u"no matching record could be found for the given ident (ident=%s)" % ident)
        
        return None



import os

from setuptools import setup, find_packages

PACKAGE_NAME = "pytoolbelt"
PACKAGE_VERSION = "0.1"

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    ]

setup(name='pytoolbelt',
      version='0.1',
      description='Python library for some commonly usefull tools',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
          "Programming Language :: Python",
          "Framework :: Pylons",
          "Topic :: Internet :: WWW/HTTP",
          "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
          ],
      author='',
      author_email='',
      url='',
      keywords='web,tools',
      package_dir={'': 'src'},
      packages=find_packages("src"),
      include_package_data=True,
      zip_safe=False,
      #      test_suite='jhi_tools2',
      install_requires=requires,
      #      entry_points = """\
      #      [paste.app_factory]
      #      main = sportpartner24:main
      #      """,
      namespace_packages=['pytoolbelt'],
      )
